@@ -26,8 +26,13 @@
 #include <gdk/gdkkeysyms.h>
 #include "idocalendarmenuitem.h"
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void     ido_calendar_menu_item_select            (GtkMenuItem        *item);
+static void     ido_calendar_menu_item_deselect          (GtkMenuItem        *item);
+#else
 static void     ido_calendar_menu_item_select            (GtkItem        *item);
 static void     ido_calendar_menu_item_deselect          (GtkItem        *item);
+#endif
 static gboolean ido_calendar_menu_item_button_release    (GtkWidget      *widget,
                                                           GdkEventButton *event);
 static gboolean ido_calendar_menu_item_button_press      (GtkWidget      *widget,
@@ -65,18 +70,27 @@ ido_calendar_menu_item_class_init (IdoCalendarMenuItemClass *klass)
   GObjectClass     *gobject_class;
   GtkWidgetClass   *widget_class;
   GtkMenuItemClass *menu_item_class;
+#if ! GTK_CHECK_VERSION (3, 0, 0)
   GtkItemClass     *item_class;
+#endif
 
   gobject_class = G_OBJECT_CLASS (klass);
   widget_class = GTK_WIDGET_CLASS (klass);
   menu_item_class = GTK_MENU_ITEM_CLASS (klass);
+#if ! GTK_CHECK_VERSION (3, 0, 0)
   item_class = GTK_ITEM_CLASS (klass);
+#endif
 
   widget_class->button_release_event = ido_calendar_menu_item_button_release;
   widget_class->button_press_event = ido_calendar_menu_item_button_press;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+  menu_item_class->select = ido_calendar_menu_item_select;
+  menu_item_class->deselect = ido_calendar_menu_item_deselect;
+#else
   item_class->select = ido_calendar_menu_item_select;
   item_class->deselect = ido_calendar_menu_item_deselect;
+#endif
 
   menu_item_class->hide_on_activate = TRUE;
 
@@ -101,12 +115,6 @@ static void
 ido_calendar_menu_item_init (IdoCalendarMenuItem *item)
 {
   IdoCalendarMenuItemPrivate *priv;
-  GtkBorder border;
-
-  border.left = 4;
-  border.right = 4;
-  border.top = 2;
-  border.bottom = 2;
 
   priv = item->priv = IDO_CALENDAR_MENU_ITEM_GET_PRIVATE (item);
 
@@ -143,7 +151,7 @@ ido_calendar_menu_item_send_focus_change (GtkWidget *widget,
     gtk_widget_grab_focus (widget);
 
   event->focus_change.type = GDK_FOCUS_CHANGE;
-  event->focus_change.window = g_object_ref (widget->window);
+  event->focus_change.window = g_object_ref (gtk_widget_get_window (widget));
   event->focus_change.in = in;
 
   gtk_widget_event (widget, event);
@@ -168,9 +176,9 @@ ido_calendar_menu_item_key_press (GtkWidget   *widget,
       gtk_widget_event (calendar,
                         ((GdkEvent *)(void*)(event)));
 
-      if (calendar->window != NULL)
+      if (gtk_widget_get_window (calendar) != NULL)
         {
-          gdk_window_raise (calendar->window);
+          gdk_window_raise (gtk_widget_get_window (calendar));
         }
 
       if (!gtk_widget_has_focus (calendar))
@@ -178,7 +186,7 @@ ido_calendar_menu_item_key_press (GtkWidget   *widget,
           gtk_widget_grab_focus (calendar);
         }
 
-      return event->keyval != GDK_Return;
+      return event->keyval != GDK_KEY_Return;
     }
 
   return FALSE;
@@ -188,43 +196,72 @@ static gboolean
 ido_calendar_menu_item_button_press (GtkWidget      *widget,
                                      GdkEventButton *event)
 {
-  GtkWidget *calendar = IDO_CALENDAR_MENU_ITEM (widget)->priv->calendar;
+	GtkWidget *calendar = IDO_CALENDAR_MENU_ITEM (widget)->priv->calendar;
 
-  if (event->button == 1)
-    {
-      if (calendar->window != NULL)
-        {
-          gdk_window_raise (calendar->window);
-        }
+	if (event->button == 1) {
+		if (gtk_widget_get_window (calendar) != NULL) {
+			gdk_window_raise (gtk_widget_get_window (calendar));
+		}
 
-      if (!gtk_widget_has_focus (calendar))
-        {
-          gtk_widget_grab_focus (calendar);
-        }
+		if (!gtk_widget_has_focus (calendar)) {
+			gtk_widget_grab_focus (calendar);
+		}
 
-      gtk_widget_event (calendar,
-                        ((GdkEvent *)(void*)(event)));
+#if GTK_CHECK_VERSION (3, 0, 0)
+		GdkEvent * newevent = gdk_event_copy((GdkEvent *)(event));
+		GList * children = gdk_window_get_children(gtk_widget_get_window(calendar));
+		GList * child;
 
-      return TRUE;
-    }
+		gint root_x = event->x_root;
+		gint root_y = event->y_root;
 
-  return FALSE;
+		for (child = children; child != NULL; child = g_list_next(child)) {
+			gint newx, newy;
+			gint winx, winy;
+			GdkWindow * newwindow = (GdkWindow*)child->data;
+
+			((GdkEventButton *)newevent)->window = newwindow;
+
+			gdk_window_get_origin(newwindow, &winx, &winy);
+			newx = root_x - winx;
+			newy = root_y - winy;
+
+			if (newx >= 0 && newy >= 0 && newx < gdk_window_get_width(newwindow) && newy < gdk_window_get_height(newwindow)) {
+				((GdkEventButton *)newevent)->x = newx;
+				((GdkEventButton *)newevent)->y = newy;
+
+				GTK_WIDGET_GET_CLASS(calendar)->button_press_event(GTK_WIDGET(calendar), (GdkEventButton*)newevent);
+			}
+		}
+
+		((GdkEventButton *)newevent)->window = event->window;
+		gdk_event_free(newevent);
+#endif
+
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 static gboolean
 ido_calendar_menu_item_button_release (GtkWidget      *widget,
                                        GdkEventButton *event)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
   GtkWidget *calendar = IDO_CALENDAR_MENU_ITEM (widget)->priv->calendar;
-
-  gtk_widget_event (calendar,
-                    ((GdkEvent *)(void*)(event)));
+  GTK_WIDGET_GET_CLASS(calendar)->button_release_event(GTK_WIDGET(calendar), event);
+#endif
 
   return TRUE;
 }
 
 static void
+#if GTK_CHECK_VERSION (3, 0, 0)
+ido_calendar_menu_item_select (GtkMenuItem *item)
+#else
 ido_calendar_menu_item_select (GtkItem *item)
+#endif
 {
   IDO_CALENDAR_MENU_ITEM (item)->priv->selected = TRUE;
 
@@ -232,7 +269,11 @@ ido_calendar_menu_item_select (GtkItem *item)
 }
 
 static void
+#if GTK_CHECK_VERSION (3, 0, 0)
+ido_calendar_menu_item_deselect (GtkMenuItem *item)
+#else
 ido_calendar_menu_item_deselect (GtkItem *item)
+#endif
 {
   IDO_CALENDAR_MENU_ITEM (item)->priv->selected = FALSE;
 
@@ -244,12 +285,12 @@ static void
 calendar_realized_cb (GtkWidget        *widget,
                       IdoCalendarMenuItem *item)
 {
-  if (widget->window != NULL)
+  if (gtk_widget_get_window (widget) != NULL)
     {
-      gdk_window_raise (widget->window);
+      gdk_window_raise (gtk_widget_get_window (widget));
     }
 
-  g_signal_connect (GTK_WIDGET (item)->parent,
+  g_signal_connect (gtk_widget_get_parent (GTK_WIDGET (item)),
                     "key-press-event",
                     G_CALLBACK (ido_calendar_menu_item_key_press),
                     item);
@@ -328,7 +369,8 @@ ido_calendar_menu_item_mark_day	(IdoCalendarMenuItem *menuitem, guint day)
 {
   g_return_val_if_fail(IDO_IS_CALENDAR_MENU_ITEM(menuitem), FALSE);
   
-  return gtk_calendar_mark_day(GTK_CALENDAR (menuitem->priv->calendar), day);
+  gtk_calendar_mark_day(GTK_CALENDAR (menuitem->priv->calendar), day);
+  return TRUE;
 }
 
 gboolean
@@ -336,7 +378,8 @@ ido_calendar_menu_item_unmark_day (IdoCalendarMenuItem *menuitem, guint day)
 {
   g_return_val_if_fail(IDO_IS_CALENDAR_MENU_ITEM(menuitem), FALSE);
   
-  return gtk_calendar_unmark_day(GTK_CALENDAR (menuitem->priv->calendar), day);
+  gtk_calendar_unmark_day(GTK_CALENDAR (menuitem->priv->calendar), day);
+  return TRUE;
 }
 
 void
