@@ -23,11 +23,18 @@
  *    Cody Russell <crussell@canonical.com>
  */
 
+#include "config.h"
+
+
 #include <gtk/gtk.h>
 #include "idorange.h"
 #include "idoscalemenuitem.h"
 #include "idotypebuiltins.h"
 
+#ifdef USE_GTK3
+#include "idooffscreenproxy.h"
+#endif
+
 static void     ido_scale_menu_item_set_property           (GObject               *object,
                                                             guint                  prop_id,
                                                             const GValue          *value,
@@ -57,6 +64,11 @@ static void     update_packing                             (IdoScaleMenuItem
 
 struct _IdoScaleMenuItemPrivate {
   GtkWidget            *scale;
+  
+#ifdef USE_GTK3
+  GtkWidget            *proxy;
+#endif
+
   GtkAdjustment        *adjustment;
   GtkWidget            *primary_image;
   GtkWidget            *secondary_image;
@@ -185,8 +197,8 @@ ido_scale_menu_item_size_allocate (GtkWidget     *widget,
       priv->right_padding = primary_padding;
     }
 
-  priv->child_allocation.x = GTK_CONTAINER (widget)->border_width + widget->style->xthickness;
-  priv->child_allocation.y = GTK_CONTAINER (widget)->border_width + widget->style->ythickness;
+  priv->child_allocation.x = gtk_container_get_border_width (GTK_CONTAINER (widget)) + gtk_widget_get_style (widget)->xthickness;
+  priv->child_allocation.y = gtk_container_get_border_width (GTK_CONTAINER (widget)) + gtk_widget_get_style (widget)->ythickness;
 
   priv->child_allocation.x += horizontal_padding;
   priv->child_allocation.x += priv->toggle_size;
@@ -213,7 +225,7 @@ ido_scale_menu_item_constructed (GObject *object)
 {
   IdoScaleMenuItem *self = IDO_SCALE_MENU_ITEM (object);
   IdoScaleMenuItemPrivate *priv = GET_PRIVATE (self);
-  GtkObject *adj = gtk_adjustment_new (0.0, 0.0, 100.0, 1.0, 10.0, 0.0);
+  GtkAdjustment *adj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 100.0, 1.0, 10.0, 0.0));
   IdoRangeStyle range_style;
   GtkWidget *hbox;
 
@@ -226,6 +238,14 @@ ido_scale_menu_item_constructed (GObject *object)
   priv->scale = ido_range_new (adj, range_style);
   g_object_ref (priv->scale);
   gtk_scale_set_draw_value (GTK_SCALE (priv->scale), FALSE);
+  
+#ifdef USE_GTK3
+  gtk_widget_set_can_focus (priv->scale, FALSE);
+  
+  priv->proxy = ido_offscreen_proxy_new ();
+  g_object_ref (priv->proxy);
+  gtk_container_add (GTK_CONTAINER (priv->proxy), priv->scale);
+#endif
 
   hbox = gtk_hbox_new (FALSE, 0);
 
@@ -261,7 +281,6 @@ static void
 ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
 {
   GObjectClass      *gobject_class = G_OBJECT_CLASS (item_class);
-  GtkObjectClass    *object_class = GTK_OBJECT_CLASS (item_class);
   GtkWidgetClass    *widget_class = GTK_WIDGET_CLASS (item_class);
 
   widget_class->button_press_event   = ido_scale_menu_item_button_press_event;
@@ -325,7 +344,7 @@ ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
                                            g_cclosure_marshal_VOID__VOID,
                                            G_TYPE_NONE, 0);
 
-  g_type_class_add_private (object_class, sizeof (IdoScaleMenuItemPrivate));
+  g_type_class_add_private (item_class, sizeof (IdoScaleMenuItemPrivate));
 }
 
 static void
@@ -339,23 +358,39 @@ update_packing (IdoScaleMenuItem *self, IdoScaleMenuItemStyle style, IdoScaleMen
       switch (old_style)
         {
         case IDO_SCALE_MENU_ITEM_STYLE_NONE:
-          gtk_container_remove (container, priv->scale);
+#ifdef USE_GTK3
+          gtk_container_remove (container, priv->proxy);
+#else 
+	  gtk_container_remove (container, priv->scale);
+#endif
           break;
 
         case IDO_SCALE_MENU_ITEM_STYLE_IMAGE:
           gtk_container_remove (container, priv->primary_image);
           gtk_container_remove (container, priv->secondary_image);
-          gtk_container_remove (container, priv->scale);
+#ifdef USE_GTK3
+          gtk_container_remove (container, priv->proxy);
+#else
+	  gtk_container_remove (container, priv->scale);
+#endif
           break;
 
         case IDO_SCALE_MENU_ITEM_STYLE_LABEL:
           gtk_container_remove (container, priv->primary_label);
           gtk_container_remove (container, priv->secondary_label);
-          gtk_container_remove (container, priv->scale);
+#ifdef USE_GTK3
+          gtk_container_remove (container, priv->proxy);
+#else
+	  gtk_container_remove (container, priv->scale);
+#endif
           break;
 
         default:
-          gtk_container_remove (container, priv->scale);
+#ifdef USE_GTK3
+          gtk_container_remove (container, priv->proxy);
+#else
+	  gtk_container_remove (container, priv->scale);
+#endif
           break;
         }
     }
@@ -363,23 +398,41 @@ update_packing (IdoScaleMenuItem *self, IdoScaleMenuItemStyle style, IdoScaleMen
   switch (style)
     {
     case IDO_SCALE_MENU_ITEM_STYLE_NONE:
+#ifdef USE_GTK3
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->proxy, FALSE, FALSE, 0);
+#else
       gtk_box_pack_start (GTK_BOX (priv->hbox), priv->scale, FALSE, FALSE, 0);
+#endif
       break;
 
     case IDO_SCALE_MENU_ITEM_STYLE_IMAGE:
       gtk_box_pack_start (GTK_BOX (priv->hbox), priv->primary_image, FALSE, FALSE, 0);
+
+#ifdef USE_GTK3
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->proxy, FALSE, FALSE, 0);
+#else
       gtk_box_pack_start (GTK_BOX (priv->hbox), priv->scale, FALSE, FALSE, 0);
+#endif
+
       gtk_box_pack_start (GTK_BOX (priv->hbox), priv->secondary_image, FALSE, FALSE, 0);
       break;
 
     case IDO_SCALE_MENU_ITEM_STYLE_LABEL:
       gtk_box_pack_start (GTK_BOX (priv->hbox), priv->primary_label, FALSE, FALSE, 0);
+#ifdef USE_GTK3
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->proxy, FALSE, FALSE, 0);
+#else
       gtk_box_pack_start (GTK_BOX (priv->hbox), priv->scale, FALSE, FALSE, 0);
+#endif
       gtk_box_pack_start (GTK_BOX (priv->hbox), priv->secondary_label, FALSE, FALSE, 0);
       break;
 
     default:
+#ifdef USE_GTK3
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->proxy, FALSE, FALSE, 0);
+#else
       gtk_box_pack_start (GTK_BOX (priv->hbox), priv->scale, FALSE, FALSE, 0);
+#endif
       break;
     }
 
@@ -470,12 +523,13 @@ ido_scale_menu_item_button_press_event (GtkWidget      *menuitem,
                                         GdkEventButton *event)
 {
   IdoScaleMenuItemPrivate *priv = GET_PRIVATE (menuitem);
-  GtkWidget *scale = priv->scale;
-  GtkWidget *parent;
   gdouble x;
+  
+#ifndef USE_GTK3
+  GtkWidget *scale = priv->scale;
+#endif
 
   // can we block emissions of "grab-notify" on parent??
-  parent = gtk_widget_get_parent (GTK_WIDGET (menuitem));
 
   translate_event_coordinates (menuitem, event->x, &x);
   event->x = x;
@@ -483,12 +537,16 @@ ido_scale_menu_item_button_press_event (GtkWidget      *menuitem,
   translate_event_coordinates (menuitem, event->x_root, &x);
   event->x_root = x;
 
+#ifndef USE_GTK3
   ubuntu_gtk_widget_set_has_grab (scale, TRUE);
+#endif
 
-  gtk_widget_event (scale,
+  gtk_widget_event (priv->scale,
                     ((GdkEvent *)(void*)(event)));
 
+#ifndef USE_GTK3
   ubuntu_gtk_widget_set_has_grab (scale, FALSE);
+#endif
 
   if (!priv->grabbed)
     {
@@ -496,7 +554,11 @@ ido_scale_menu_item_button_press_event (GtkWidget      *menuitem,
       g_signal_emit (menuitem, signals[SLIDER_GRABBED], 0);
     }
 
+#ifdef USE_GTK3
+  return FALSE;
+#else
   return TRUE;
+#endif
 }
 
 static gboolean
@@ -505,7 +567,6 @@ ido_scale_menu_item_button_release_event (GtkWidget *menuitem,
 {
   IdoScaleMenuItemPrivate *priv = GET_PRIVATE (menuitem);
   GtkWidget *scale = priv->scale;
-  GdkWindow *tmp = event->window;
   gdouble x;
 
   if (event->x > priv->child_allocation.x &&
@@ -554,8 +615,6 @@ ido_scale_menu_item_button_release_event (GtkWidget *menuitem,
       return TRUE;
     }
 
-  event->window = GTK_RANGE (scale)->event_window;
-
   translate_event_coordinates (menuitem, event->x, &x);
   event->x = x;
 
@@ -565,8 +624,6 @@ ido_scale_menu_item_button_release_event (GtkWidget *menuitem,
   gtk_widget_event (scale,
                     ((GdkEvent *)(void*)(event)));
 
-  event->window = tmp;
-
   if (priv->grabbed)
     {
       priv->grabbed = FALSE;
@@ -688,13 +745,13 @@ ido_scale_menu_item_new_with_range (const gchar  *label,
                                     gdouble       max,
                                     gdouble       step)
 {
-  GtkObject *adjustment = gtk_adjustment_new (value, min, max, step, 10 * step, 0);
+  GtkAdjustment *adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (value, min, max, step, 10 * step, 0));
 
-  return g_object_new (IDO_TYPE_SCALE_MENU_ITEM,
-                       "label",       label,
-                       "range-style", range_style,
-                       "adjustment",  adjustment,
-		       NULL);
+  return GTK_WIDGET (g_object_new (IDO_TYPE_SCALE_MENU_ITEM,
+                                  "label",       label,
+                                  "range-style", range_style,
+                                  "adjustment",  adjustment,
+                     NULL));
 }
 
 /**
@@ -804,7 +861,7 @@ ido_scale_menu_item_get_secondary_image (IdoScaleMenuItem *menuitem)
  * Whether this is visible depends upon the return value from
  * ido_scale_menu_item_get_style().
  **/
-G_CONST_RETURN gchar*
+const gchar*
 ido_scale_menu_item_get_primary_label (IdoScaleMenuItem *menuitem)
 {
   IdoScaleMenuItemPrivate *priv;
@@ -825,7 +882,7 @@ ido_scale_menu_item_get_primary_label (IdoScaleMenuItem *menuitem)
  * Whether this is visible depends upon the return value from
  * ido_scale_menu_item_get_style().
  **/
-G_CONST_RETURN gchar*
+const gchar*
 ido_scale_menu_item_get_secondary_label (IdoScaleMenuItem *menuitem)
 {
   IdoScaleMenuItemPrivate *priv;
